import obt.deco
import obt.path
import obt.template
from obt.command import Command
import fileinput

deco = obt.deco.Deco()

class patcher:

  def __init__(self,provider,repl_dict=None):
    name = provider._name
    self._ori = obt.path.patches(provider)/name/"ori"
    self._chg = obt.path.patches(provider)/name/"chg"
    self._repl_dict = repl_dict

  def patch(self,dest_dir,file):
    src  = self._chg/file
    dest = dest_dir/file
    print("Patching <%s -> %s>" % (deco.bright(src), deco.yellow(dest)))
    Command(["cp","-f",src,dest]).exec()
    if self._repl_dict:
      obt.template.template_file(dest,self._repl_dict)

  def patch_list(self,list_of_items):
    for i in list_of_items:
      print(i)
      self.patch(i[0],i[1])


def patch_with_dict(filename,item_dict):
  for k in item_dict.keys():
    v = item_dict[k]
    with fileinput.FileInput(str(filename), inplace=True, backup='.bak') as file:
      for line in file:
        print(line.replace(k, v), end='')



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "master"

import os, tarfile
from obt import dep, host, path, cmake, git, make, command
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################

class moltenvk(dep.Provider):

  def __init__(self): ############################################
    super().__init__("moltenvk")
    #print(options)
    self.source_root = path.builds()/"moltenvk"
    self.build_dest = path.builds()/"moltenvk"/".build"
    self.manifest = path.manifests()/"moltenvk"
    self.OK = self.manifest.exists()
    self._archlist = ["x86_64"]
    self._oslist = ["Darwin"]

  def __str__(self): ##########################################################

    return "MoltenVK (github-%s)" % VERSION

  def wipe(self): #############################################################
    os.system("rm -rf %s"%self.source_root)
    os.system("rm -rf %s"%self.build_dest)

  def build(self): ##########################################################

    #glfw = dep.require("glfw")

    if not self.source_root.exists():
      git.Clone("https://github.com/KhronosGroup/MoltenVK",self.source_root,VERSION)

    os.chdir(self.source_root)

    command.system(["./fetchDependencies"])
    cmd = ["make", "macos"]
    ok = (0 == command.system(cmd))
    if ok:
      cmd = ["cp","Package/Latest/MoltenVK/macOS/static/libMoltenVK.a",path.libs()/"libMoltenVK.a"]
      ok = (0 == command.system(cmd))
      if ok:
        cmd = ["cp","-r","Package/Latest/MoltenVK/include/*",path.includes()]
        ok = (0 == command.system(cmd))
    return ok



from obt import host, dep, path, osrelease

###############################################################################

class cuda(dep.Provider):

  def __init__(self): ############################################
    super().__init__("cuda")
    self.manifest = path.manifests()/"cuda"
    self.OK = self.manifest.exists()

  ########

  def __str__(self):
    return "cuda (system)"

  ########

  @property
  def cxx_compiler(self):
    if host.IsDebian:
      desc = host.description()
      if desc.codename=="jammy":
        return "g++-9"
      else:
        return "g++-8"
    return "g++"
  @property
  def c_compiler(self):
    if host.IsDebian:
      desc = host.description()
      if desc.codename=="jammy":
        return "gcc-9"
      else:
        return "gcc-8"
    return "gcc"

  ########

  #wget http://developer.download.nvidia.com/compute/cuda/11.0.2/local_installers/cuda_11.0.2_450.51.05_linux.run


  def provide(self): ##########################################################
    self.manifest.touch()
    return True

  def areRequiredSourceFilesPresent(self):
    return path.Path("/usr/bin/nvcc").exists()
  def areRequiredBinaryFilesPresent(self):
    return path.Path("/usr/bin/nvcc").exists()


#/usr/local/cuda-10.2



from obt import dep, pip, path, dep, pathtools, command, log, env
Command = command.Command
from yarl import URL
###############################################################################
VERSION = "2023.1"
###############################################################################

class vivado(dep.Provider):

  def __init__(self): ############################################
    super().__init__("vivado")
    build_dest = path.builds()/"vivado"
    self.build_dest = build_dest
    self.manifest = path.manifests()/"vivado"
    self.OK = self.manifest.exists()
    self.vivbase = path.vivado_base()/VERSION

  ########

  def base(self):
      return self.vivbase

  def build(self): ############################################################
      return 0

  ########

  def env_init(self):
    log.marker("registering vivado(%s) SDK"%VERSION)
    env.append("PATH",self.vivbase/"bin")

  ########

  def on_build_shell(self):
    pathtools.mkdir(self.build_dest,clean=False)
    return command.subshell( directory=self.build_dest,
                             prompt = "LITEX",
                             environment = dict() )

  ########

  def areRequiredSourceFilesPresent(self):
    return (self.vivbase/"bin"/"vivado").exists()

  def areRequiredBinaryFilesPresent(self):
    return None



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, host, command, path, pathtools

###############################################################################

class glfw(dep.StdProvider):
  name = "glfw"
  def __init__(self):
    super().__init__(glfw.name)
    ###########################################
    ## GLFW installs with wrong rpath install name
    ##  on mac, so we fix it up here..
    ###########################################
    class FixOsx(dep.CMakeBuilder):
      def __init__(self,name):
        super().__init__(name)
      def install(self,blddir):
        success = super().install(blddir)
        if success and host.IsOsx:
          dylibname = "libglfw.3.dylib"
          retc = command.run(["install_name_tool","-id",
                              "@rpath/"+dylibname,path.libs()/dylibname])
          success = retc==0
        return success
    ###########################################
    builder_class = dep.switch(linux=dep.CMakeBuilder,
                               macos=FixOsx)
    
    if builder_class == dep.CMakeBuilder:
      self.declareDep("cmake")


    self._builder = builder_class(glfw.name)
    
    def post_install():
      print("post installing GLFW!!")
      glad_srcdir = path.builds()/"glfw"/"deps"/"glad"
      glad_dstdir = path.includes()/"glad"
      pathtools.ensureDirectoryExists(glad_dstdir)
      for item in ["gl.h","khrplatform.h","vk_platform.h","vulkan.h"]:
        pathtools.copyfile(glad_srcdir/item,glad_dstdir/item)
      pass 

    self._builder._onPostInstall = post_install
    #self._builder.setCmVar("GLFW_VULKAN_STATIC","TRUE")
    ###########################################
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=glfw.name,
                             repospec="glfw/glfw",
                             revision="216d5e8402513b582563d5b8433fefb449a1593e",
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return path.decorate_obt_lib("glfw").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os

import obt.deco
from obt import log

deco = obt.deco.Deco()

###########################################

def set(key,val):

  #log.output(deco.orange("set")+" var<" + deco.key(str(key))+"> to <" + deco.path(val) + ">")
  os.environ[str(key)] = str(val)

###########################################

def prepend(key,val):
  if False==(str(key) in os.environ):
    set(key,str(val))
  else:
    os.environ[str(key)] = str(val) + ":" + os.environ[key]
    #log.output(deco.magenta("prepend")+" var<" + deco.key(key) + "> to<" + deco.path(os.environ[key]) + ">")

###########################################

def append(key,val):
  val = os.path.normpath(str(val))
  if False==(str(key) in os.environ):
    set(key,val)
  else:
    os.environ[str(key)] = os.environ[str(key)]+":"+str(val)
    #log.output(deco.cyan("append")+" var<" + deco.key(key) + "> to<" + deco.path(os.environ[key]) + ">")



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from obt import dep, host, path, git, make, pathtools, env, command
from obt.deco import Deco
from obt.wget import wget
from obt.cmake import context as cmake_context
from obt import log
from yarl import URL
from pathlib import Path

deco = Deco()
cmd = command.Command 

###############################################################################

class _qt5_from_source(dep.Provider):


  def __init__(self): ############################################
    super().__init__("qt5")
    #print(options)
    self.MAJOR_VERSION = "5.15"
    self.MINOR_VERSION = "2"
    self.HASH = "e1447db4f06c841d8947f0a6ce83a7b5"
    self.manifest = path.manifests()/"qt5"
    self.OK = self.manifest.exists()
    self.baseurl = URL("https://mirrors.ukfast.co.uk/sites/qt.io/official_releases/qt")
    self.fullver = "%s.%s" % (self.MAJOR_VERSION,self.MINOR_VERSION)
    self.name = "qt-everywhere-src-%s" % self.fullver
    self.xzname = "%s.tar.xz" % self.name
    self.url = self.baseurl/self.MAJOR_VERSION/self.fullver/"single"/self.xzname
    self.source_base = path.builds()/"qt5"
    self.source_root = self.source_base/self.name
    self.build_dest = path.builds()/"qt5"/"qt5-build"
    self._archlist = ["x86_64"]
    self.declareDep("assimp")
  ########
  def env_goto(self):
    return {
      "qt5-src": self.source_root,
      "qt5-build": self.build_dest
    }
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"README").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.qt5dir()/"bin"/"qmlscene").exists()
  ########
  def on_build_shell(self):
    return command.subshell( directory=self.build_dest,
                             prompt = "QT5",
                             environment = dict() )
  ########
  def download_and_extract(self): #############################################
    self.arcpath = dep.downloadAndExtract([self.url],
                                           self.xzname,
                                           "xz",
                                           self.HASH,
                                           self.source_base)
  ########
  def wipe(self):
    os.system("rm -rf %s"%self.source_root)
    os.system("rm -rf %s"%self.build_dest)
  ########
  def build(self): ############################################################
    if dep.require(["assimp"])==None:
      return False
    self.OK = True
    #########################################
    # fetch source
    #########################################
    if not self.source_root.exists():
        self.download_and_extract()
    #########################################
    # prep for build
    #########################################
    if self.should_incremental_build:
        os.chdir(self.build_dest)
    else:
        pathtools.mkdir(self.build_dest,clean=True)
        os.chdir(self.build_dest)

        options =  ["-prefix", path.qt5dir()]
        options += ["-release"]
        options += ["-opensource", "-confirm-license"]
        #options += ["-c++std", "c++14", "-shared"]
        #options += ["-nomake", "tests"]
        #options += ["-nomake", "examples"]
        #options += ["-opengl","desktop"]

        if host.IsOsx:
          print("yo")
          #options += ["-qt-libpng","-qt-zlib","-no-framework.]
          #options += [,"-qt-libjpeg","-qt-pcre","-qt-freetype"]
          #options += ["-no-feature-sql"] # xcode11 + sql-tds (long long / uint64_t typedef compile errors)
          #options += ["-no-feature-location"]
        else:
          options += ["-system-zlib"]
          options += ["-xcb"]
        options += ["-skip","qtwebengine"]
        options += ["-system-assimp"]
        #options += ["-no-rpath"]
        #options += ["-pkg-config"]
        #options += ["-proprietary-codecs"]

        b = cmd(["sh", self.source_root/"configure"]+options)
        self.OK = (b.exec()==0)
    #########################################
    # build
    #########################################
    if self.OK:
      self.OK = (make.exec(parallelism=self.default_parallelism)==0)
    if self.OK:
      self.OK = (make.exec(parallelism=self.default_parallelism)==0)
    # uhhuh - https://bugreports.qt.io/browse/QTBUG-60496
    if self.OK:
      self.OK = (0==make.exec(target="install", parallelism=0.0))
    return self.OK

###############################################################################

class _qt5_from_homebrew(dep.HomebrewProvider):
  def __init__(self):
    super().__init__("qt5","qt5")
    self.fullver = "5.15.1"
  def install_dir(self):
    return path.Path("/usr/local/opt/qt5")

###############################################################################

BASE = _qt5_from_source
if host.IsOsx:
  BASE = _qt5_from_homebrew

###############################################################################

class qt5(BASE):
  def __init__(self):
    super().__init__()
  ########
  def __str__(self):
    return "QT5"
  ########
  def env_init(self):
    log.marker("registering QT5(%s) SDK"%self.fullver)
    if host.IsOsx:
      qtdir = Path("/")/"usr"/"local"/"opt"/"qt5"
    else:
      qtdir = path.stage()/"qt5"
    env.set("QTDIR",qtdir)
    env.prepend("PATH",qtdir/"bin")
    env.prepend("LD_LIBRARY_PATH",qtdir/"lib")
    #env.append("PKG_CONFIG_PATH",qtdir/"lib"/"pkgconfig")
    env.prepend("PKG_CONFIG_PATH",qtdir/"lib"/"pkgconfig")
    env.set("QTVER",self.fullver)
  ########
  @property
  def include_dir(self):
    return path.qt5dir()/"include"



#!/usr/bin/env python3

import os, sys, pathlib, argparse
from obt import dep, host, path
import obt._globals
import obt.deco
deco = obt.deco.Deco()

parser = argparse.ArgumentParser(description='obt.build dep build shell')
parser.add_argument('dependency', metavar='D', type=str, help='a dependency to build')

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

depname = _args["dependency"]
DEP = dep.instance(depname)
if DEP==None:
   print(deco.red("Dep<%s> not found!"%depname))
   sys.exit(0)

if not hasattr(DEP,"on_build_shell"):
   print(deco.red("Dep<%s> does not support on_build_shell!"%depname))
   sys.exit(0)

print(deco.bright("Entering build shell for Dep<%s>! just exit to return here"%depname))

retc = DEP.on_build_shell()
if retc==None:
  retc = 0

sys.exit(retc)



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile, sys
from obt import dep, host, path, git, cmake, make
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

import _binutils

deco = Deco()

###############################################################################

class avr_binutils(dep.Provider):

  def __init__(self): ############################################

    super().__init__("avr_binutils")

    self.manifest = path.manifests()/"avr_binutils"
    self.OK = self.manifest.exists()
    self._archlist = ["x86_64"]

  ########

  def __str__(self):
    return "Avr BinUtils (source)"

  ########

  def provide(self): ##########################################################

    if False==self.OK:
        bu = _binutils.context("binutils-avr")
        bdest = bu.build_dir/".build"

        os.mkdir(bdest)
        os.chdir(bdest)

        Command(['../configure',
                 '--prefix=%s'%path.prefix(),
                 '--target=avr',
                 '--disable-werror',
                 '--disable-nls']).exec()

        make.exec("all")
        make.exec("install",parallelism=0.0)
        self.manifest.touch()

    self.OK = True
    return self.OK



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import argparse, sys, os
from obt import deco, path
from obt.eda.xilinx import vivado

deco = deco.Deco()

epilog =  deco.orange("Example (vivado help): %s --batch -- -help\n"%sys.argv[0])
epilog += deco.orange("Example (vivado tcl REPL): %s --tcl"%sys.argv[0])
epilog += deco.orange("Example (user shell command): %s --exec -- lsb_release -a"%sys.argv[0])

parser = argparse.ArgumentParser(description='Launch command in EDA Docker Containers',
                                 epilog=epilog)
parser.add_argument('--gui', action="store_true", help=deco.yellow('Launch Vivado GUI'))
parser.add_argument('--batch', action="store_true", help=deco.yellow('Launch Vivado in batch mode, seperate vivado arguments using -- arg break'))
parser.add_argument('--tcl', action="store_true", help=deco.yellow('Launch Vivado in tcl-shell mode'))
parser.add_argument('--shell', action="store_true", help=deco.yellow('Launch Vivado/LiteX enabled bash'))
parser.add_argument('--exec', action="store_true", help=deco.yellow('exec command in Vivado/Litex container'))
parser.add_argument('--posttag', help=deco.yellow('tag container with <name> post execution'))
parser.add_argument('remainderargs', nargs=argparse.REMAINDER)

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

cwd = path.Path(os.getcwd())
vctx = vivado.Context(hostdir=cwd)
tag = None

if _args["posttag"]:
  tag = path.Path(_args["posttag"])

if _args["gui"]:
  vctx.env["_JAVA_OPTIONS"] = "-Dawt.useSystemAAFontSettings=on "\
                              "-Dswing.aatext=true "\
                              "-Dswing.defaultlaf=com.sun.java.swing.plaf.gtk.GTKLookAndFeel "\
                              "-Dswing.crossplatformlaf=com.sun.java.swing.plaf.gtk.GTKLookAndFeel"
  vctx.run(args=[],posttag=tag)
elif _args["tcl"]:
  vctx.run(interactive=True,args=["-mode","tcl","-nojournal","-nolog"],posttag=tag)
elif _args["batch"]:
  remargs = _args["remainderargs"][1:]
  vctx.run_batch(args=remargs,posttag=tag)
elif _args["shell"]:
  vctx.shell()
elif _args["exec"]:
  remargs = _args["remainderargs"][1:]
  vctx.shell_command(remargs,posttag=tag)
else:
  assert(False)



#!/usr/bin/env python3 

from __future__ import print_function
'''
Basic Multi GPU computation example using TensorFlow library.

Author: Aymeric Damien
Project: https://github.com/aymericdamien/TensorFlow-Examples/
'''

'''
This tutorial requires your machine to have 2 GPUs
"/cpu:0": The CPU of your machine.
"/gpu:0": The first GPU of your machine
"/gpu:1": The second GPU of your machine
'''



import numpy as np
import tensorflow as tf
import datetime

# Processing Units logs
log_device_placement = True

# Num of multiplications to perform
n = 10

'''
Example: compute A^n + B^n on 2 GPUs
Results on 8 cores with 2 GTX-980:
 * Single GPU computation time: 0:00:11.277449
 * Multi GPU computation time: 0:00:07.131701
'''
# Create random large matrix
A = np.random.rand(10000, 10000).astype('float32')
B = np.random.rand(10000, 10000).astype('float32')

# Create a graph to store results
c1 = []
c2 = []

def matpow(M, n):
    if n < 1: #Abstract cases where n < 1
        return M
    else:
        return tf.matmul(M, matpow(M, n-1))

tf.compat.v1.disable_eager_execution()

'''
Single GPU computing
'''
with tf.device('/gpu:0'):
    a = tf.compat.v1.placeholder(tf.float32, [10000, 10000])
    b = tf.compat.v1.placeholder(tf.float32, [10000, 10000])
    # Compute A^n and B^n and store results in c1
    c1.append(matpow(a, n))
    c1.append(matpow(b, n))

with tf.device('/cpu:0'):
  sum = tf.add_n(c1) #Addition of all elements in c1, i.e. A^n + B^n

t1_1 = datetime.datetime.now()
config = tf.compat.v1.ConfigProto(log_device_placement=log_device_placement)
with tf.compat.v1.Session(config=config) as sess:
    # Run the op.
    sess.run(sum, {a:A, b:B})
t2_1 = datetime.datetime.now()


'''
Multi GPU computing
'''
# GPU:0 computes A^n
with tf.device('/gpu:0'):
    # Compute A^n and store result in c2
    a = tf.compat.v1.placeholder(tf.float32, [10000, 10000])
    c2.append(matpow(a, n))

# GPU:1 computes B^n
with tf.device('/gpu:1'):
    # Compute B^n and store result in c2
    b = tf.compat.v1.placeholder(tf.float32, [10000, 10000])
    c2.append(matpow(b, n))

with tf.device('/cpu:0'):
  sum = tf.add_n(c2) #Addition of all elements in c2, i.e. A^n + B^n

t1_2 = datetime.datetime.now()
config = tf.compat.v1.ConfigProto(log_device_placement=log_device_placement)
with tf.compat.v1.Session(config=config) as sess:
    # Run the op.
    sess.run(sum, {a:A, b:B})
t2_2 = datetime.datetime.now()


print("Single GPU computation time: " + str(t2_1-t1_1))
print("Multi GPU computation time: " + str(t2_2-t1_2))



import os, inspect, tarfile
from pathlib import Path
import importlib.util
import obt.path, obt.host
from obt.command import Command, run
from obt.deco import Deco
from obt.wget import wget
from obt import pathtools, cmake, make, path, git, host

deco = Deco()

###############################################################################

def downloadAndExtract(urls,
                       outname,
                       archive_type,
                       md5val,
                       build_dest,
                       arc_options=[]):

  arcpath = wget( urls = urls,
                  output_name = outname,
                  md5val = md5val )


  if arcpath:
    if build_dest.exists():
      Command(["rm","-rf",build_dest]).exec()
    print("extracting<%s> to build_dest<%s>"%(deco.path(arcpath),deco.path(build_dest)))
    print(archive_type)
    build_dest.mkdir()
    if( archive_type=="zip" ):
        os.chdir(str(build_dest))
        Command(["unzip"]+arc_options+[arcpath]).exec()
    elif archive_type=="tgz":
        os.chdir(str(build_dest))
        Command(["tar","xvf",arcpath]).exec()
    elif archive_type=="none":
        os.chdir(str(build_dest))
        run(["cp",arcpath,build_dest/outname],do_log=True)
        pass
    else:
        print(arcpath)
        #assert(tarfile.is_tarfile(str(arcpath)))
        tf = tarfile.open(str(arcpath),mode='r:%s'%archive_type)
        tf.extractall(path=str(build_dest))

  return arcpath
